@@ -47,7 +47,7 @@ from torchness.comoneural.batcher import Batcher
 from torchness.types import TNS, DTNS
 from torchness.base_elements import mrg_ckpts
 from torchness.scaled_LR import ScaledLR
-from torchness.grad_clipping import GradClipperAVT
+from torchness.grad_clipping import GradClipperMAVG
 from torchness.tbwr import TBwr
 
 
@@ -135,11 +135,11 @@ class MOTorch(ParaSave, torch.nn.Module):
         'ann_base':         None,
         'ann_step':         1.0,
         'n_wup_off':        2.0,
-            # gradients clipping parameters (check torchness.base_elements.GradClipperAVT)
-        'clip_value':       None,
-        'avt_SVal':         0.1,
-        'avt_window':       100,
-        'avt_max_upd':      1.5,
+            # gradients clipping parameters (check torchness.base_elements.GradClipperMAVG)
+        'gc_factor':        0.01,
+        'gc_first_avg':     True,
+        'gc_start_val':     0.1,
+        'gc_max_upd':       1.5,
         'do_clip':          False,
             # other
         'try_load_ckpt':    True,               # tries to load a checkpoint while init
@@ -245,10 +245,14 @@ class MOTorch(ParaSave, torch.nn.Module):
         # INFO: device is a special parameter, MOTorch allows it to be in DevicesPypaq type - it needs to be casted to torch namespace
         self._log.debug(f'> {self.name} resolves devices, given: {self._point["device"]}')
         self._log.debug(f'>> torch.cuda.is_available(): {torch.cuda.is_available()}')
-        device = get_devices(
+        devices = get_devices(
             devices=            self._point["device"],
             torch_namespace=    True,
-            logger=             get_child(self._log, 'get_devices'))[0]
+            logger=             get_child(self._log, 'get_devices'))
+        if not devices:
+            self._log.warning(f'given device: {self._point["device"]} is not available, using CPU')
+            devices = ['cpu']
+        device = devices[0]
         self._log.info(f'> {self.name} given devices: {self._point["device"]}, will use: {device}')
 
         self._point.update({
@@ -304,7 +308,7 @@ class MOTorch(ParaSave, torch.nn.Module):
         self._module = self.module_type(**self._point_module) # private not to be saved with point
 
         if self.try_load_ckpt:
-            self.load_ckpt() # TODO do we want to do sth with returned additional data?
+            self.load_ckpt()
         else:
             self._log.info(f'> {self.name} checkpoint not loaded, not even tried because \'try_load_ckpt\' was set to {self.try_load_ckpt}')
 
@@ -334,14 +338,13 @@ class MOTorch(ParaSave, torch.nn.Module):
             n_wup_off=      self.n_wup_off,
             logger=         get_child(self._log, 'ScaledLR'))
 
-        self._grad_clipper = GradClipperAVT(
+        self._grad_clipper = GradClipperMAVG(
             module=         self,
-            clip_value=     self.clip_value,
-            avt_SVal=       self.avt_SVal,
-            avt_window=     self.avt_window,
-            avt_max_upd=    self.avt_max_upd,
-            do_clip=        self.do_clip,
-            logger=         get_child(self._log, 'GradClipperAVT'))
+            factor=         self.gc_factor,
+            first_avg=      self.gc_first_avg,
+            start_val=      self.gc_start_val,
+            max_upd=        self.gc_max_upd,
+            do_clip=        self.do_clip)
 
         # MOTorch by default is not in training mode
         self.train(False)
@@ -435,7 +438,7 @@ class MOTorch(ParaSave, torch.nn.Module):
             **kwargs)
 
         out['loss'].backward()              # update gradients
-        gnD = self._grad_clipper.clip()     # clip gradients, adds: 'gg_norm' & 'gg_avt_norm' to out
+        gnD = self._grad_clipper.clip()     # clip gradients, adds: 'gg_norm' & 'gg_norm_clip' to out
         self._opt.step()                    # apply optimizer
         self._opt.zero_grad()               # clear gradients
         self._scheduler.step()              # apply LR scheduler
@@ -502,7 +505,14 @@ class MOTorch(ParaSave, torch.nn.Module):
 
     # saves MOTorch (ParaSave POINT and model checkpoint)
     def save(self):
-        if self.read_only: raise MOTorchException('read_only MOTorch cannot be saved!')
+
+        if self.read_only:
+            raise MOTorchException('read_only MOTorch cannot be saved!')
+
+        # to properly start grad clipping after load
+        self['gc_first_avg'] = False
+        self['gc_start_val'] = self._grad_clipper.gg_norm_clip
+
         self.save_point()
         self.save_ckpt()
         self._log.info(f'{self.__class__.__name__} {self.name} saved to {self.save_topdir}')
@@ -594,7 +604,7 @@ class MOTorch(ParaSave, torch.nn.Module):
             ratio: float=                           0.5,
             noise: float=                           0.03,
             logger=                                 None,
-            loglevel=                               20,
+            loglevel=                               30,
     ) -> None:
 
         if not save_topdir_parent_main: save_topdir_parent_main = cls.SAVE_TOPDIR
@@ -715,14 +725,14 @@ class MOTorch(ParaSave, torch.nn.Module):
             batch_IX += 1
 
             if self.do_TB:
-                self.log_TB(value=loss,               tag='tr/loss',   step=self.train_step)
-                self.log_TB(value=out['gg_norm'],     tag='tr/gn',     step=self.train_step)
-                self.log_TB(value=out['gg_avt_norm'], tag='tr/gn_avt', step=self.train_step)
-                self.log_TB(value=out['currentLR'],   tag='tr/cLR',    step=self.train_step)
+                self.log_TB(value=loss,                tag='tr/loss',    step=self.train_step)
+                self.log_TB(value=out['gg_norm'],      tag='tr/gn',      step=self.train_step)
+                self.log_TB(value=out['gg_norm_clip'], tag='tr/gn_clip', step=self.train_step)
+                self.log_TB(value=out['currentLR'],    tag='tr/cLR',     step=self.train_step)
                 if acc is not None:
-                    self.log_TB(value=acc,            tag='tr/acc',    step=self.train_step)
+                    self.log_TB(value=acc,             tag='tr/acc',     step=self.train_step)
                 if f1 is not None:
-                    self.log_TB(value=f1,             tag='tr/F1',     step=self.train_step)
+                    self.log_TB(value=f1,              tag='tr/F1',      step=self.train_step)
 
             if acc is not None: tr_accL.append(acc)
             if f1 is not None: tr_f1L.append(f1)
